@@ -5,6 +5,7 @@ namespace Doctrine\Common\Annotations;
 use Doctrine\Common\Annotations\Annotation\Attribute;
 use Doctrine\Common\Annotations\Annotation\Attributes;
 use Doctrine\Common\Annotations\Annotation\Enum;
+use Doctrine\Common\Annotations\Annotation\NamedArgumentConstructor;
 use Doctrine\Common\Annotations\Annotation\Target;
 use ReflectionClass;
 use ReflectionException;
@@ -14,8 +15,11 @@ use stdClass;
 
 use function array_keys;
 use function array_map;
+use function array_pop;
+use function array_values;
 use function class_exists;
 use function constant;
+use function count;
 use function defined;
 use function explode;
 use function gettype;
@@ -24,7 +28,6 @@ use function in_array;
 use function interface_exists;
 use function is_array;
 use function is_object;
-use function is_subclass_of;
 use function json_encode;
 use function ltrim;
 use function preg_match;
@@ -145,33 +148,35 @@ final class DocParser
      */
     private static $annotationMetadata = [
         Annotation\Target::class => [
-            'is_annotation'    => true,
-            'has_constructor'  => true,
-            'properties'       => [],
-            'targets_literal'  => 'ANNOTATION_CLASS',
-            'targets'          => Target::TARGET_CLASS,
-            'default_property' => 'value',
-            'attribute_types'  => [
+            'is_annotation'                  => true,
+            'has_constructor'                => true,
+            'has_named_argument_constructor' => false,
+            'properties'                     => [],
+            'targets_literal'                => 'ANNOTATION_CLASS',
+            'targets'                        => Target::TARGET_CLASS,
+            'default_property'               => 'value',
+            'attribute_types'                => [
                 'value'  => [
-                    'required'  => false,
-                    'type'      => 'array',
+                    'required'   => false,
+                    'type'       => 'array',
                     'array_type' => 'string',
-                    'value'     => 'array<string>',
+                    'value'      => 'array<string>',
                 ],
             ],
         ],
         Annotation\Attribute::class => [
-            'is_annotation'    => true,
-            'has_constructor'  => false,
-            'targets_literal'  => 'ANNOTATION_ANNOTATION',
-            'targets'          => Target::TARGET_ANNOTATION,
-            'default_property' => 'name',
-            'properties'       => [
+            'is_annotation'                  => true,
+            'has_constructor'                => false,
+            'has_named_argument_constructor' => false,
+            'targets_literal'                => 'ANNOTATION_ANNOTATION',
+            'targets'                        => Target::TARGET_ANNOTATION,
+            'default_property'               => 'name',
+            'properties'                     => [
                 'name'      => 'name',
                 'type'      => 'type',
                 'required'  => 'required',
             ],
-            'attribute_types'  => [
+            'attribute_types'                => [
                 'value'  => [
                     'required'  => true,
                     'type'      => 'string',
@@ -190,13 +195,14 @@ final class DocParser
             ],
         ],
         Annotation\Attributes::class => [
-            'is_annotation'    => true,
-            'has_constructor'  => false,
-            'targets_literal'  => 'ANNOTATION_CLASS',
-            'targets'          => Target::TARGET_CLASS,
-            'default_property' => 'value',
-            'properties'       => ['value' => 'value'],
-            'attribute_types'  => [
+            'is_annotation'                  => true,
+            'has_constructor'                => false,
+            'has_named_argument_constructor' => false,
+            'targets_literal'                => 'ANNOTATION_CLASS',
+            'targets'                        => Target::TARGET_CLASS,
+            'default_property'               => 'value',
+            'properties'                     => ['value' => 'value'],
+            'attribute_types'                => [
                 'value' => [
                     'type'      => 'array',
                     'required'  => true,
@@ -206,13 +212,14 @@ final class DocParser
             ],
         ],
         Annotation\Enum::class => [
-            'is_annotation'    => true,
-            'has_constructor'  => true,
-            'targets_literal'  => 'ANNOTATION_PROPERTY',
-            'targets'          => Target::TARGET_PROPERTY,
-            'default_property' => 'value',
-            'properties'       => ['value' => 'value'],
-            'attribute_types'  => [
+            'is_annotation'                  => true,
+            'has_constructor'                => true,
+            'has_named_argument_constructor' => false,
+            'targets_literal'                => 'ANNOTATION_PROPERTY',
+            'targets'                        => Target::TARGET_PROPERTY,
+            'default_property'               => 'value',
+            'properties'                     => ['value' => 'value'],
+            'attribute_types'                => [
                 'value' => [
                     'type'      => 'array',
                     'required'  => true,
@@ -223,6 +230,16 @@ final class DocParser
                 ],
             ],
         ],
+        Annotation\NamedArgumentConstructor::class => [
+            'is_annotation'                  => true,
+            'has_constructor'                => false,
+            'has_named_argument_constructor' => false,
+            'targets_literal'                => 'ANNOTATION_CLASS',
+            'targets'                        => Target::TARGET_CLASS,
+            'default_property'               => null,
+            'properties'                     => [],
+            'attribute_types'                => [],
+        ],
     ];
 
     /**
@@ -484,10 +501,11 @@ final class DocParser
             self::$metadataParser->setIgnoreNotImportedAnnotations(true);
             self::$metadataParser->setIgnoredAnnotationNames($this->ignoredAnnotationNames);
             self::$metadataParser->setImports([
-                'enum'          => Annotation\Enum::class,
-                'target'        => Annotation\Target::class,
-                'attribute'     => Annotation\Attribute::class,
-                'attributes'    => Annotation\Attributes::class,
+                'enum'                     => Enum::class,
+                'target'                   => Target::class,
+                'attribute'                => Attribute::class,
+                'attributes'               => Attributes::class,
+                'namedargumentconstructor' => NamedArgumentConstructor::class,
             ]);
 
             // Make sure that annotations from metadata are loaded
@@ -495,6 +513,7 @@ final class DocParser
             class_exists(Target::class);
             class_exists(Attribute::class);
             class_exists(Attributes::class);
+            class_exists(NamedArgumentConstructor::class);
         }
 
         $class      = new ReflectionClass($name);
@@ -514,14 +533,8 @@ final class DocParser
             'is_annotation'    => strpos($docComment, '@Annotation') !== false,
         ];
 
-        if (PHP_VERSION_ID < 80000 && $class->implementsInterface(NamedArgumentConstructorAnnotation::class)) {
-            foreach ($constructor->getParameters() as $parameter) {
-                $metadata['constructor_args'][$parameter->getName()] = [
-                    'position' => $parameter->getPosition(),
-                    'default' => $parameter->isOptional() ? $parameter->getDefaultValue() : null,
-                ];
-            }
-        }
+        $metadata['has_named_argument_constructor'] = $metadata['has_constructor']
+            && $class->implementsInterface(NamedArgumentConstructorAnnotation::class);
 
         // verify that the class is really meant to be an annotation
         if ($metadata['is_annotation']) {
@@ -535,6 +548,14 @@ final class DocParser
                     continue;
                 }
 
+                if ($annotation instanceof NamedArgumentConstructor) {
+                    $metadata['has_named_argument_constructor'] = $metadata['has_constructor'];
+                    if ($metadata['has_named_argument_constructor']) {
+                        // choose the first argument as the default property
+                        $metadata['default_property'] = $constructor->getParameters()[0]->getName();
+                    }
+                }
+
                 if (! ($annotation instanceof Attributes)) {
                     continue;
                 }
@@ -581,7 +602,7 @@ final class DocParser
                         }
 
                         $metadata['enum'][$property->name]['value']   = $annotation->value;
-                        $metadata['enum'][$property->name]['literal'] = ( ! empty($annotation->literal))
+                        $metadata['enum'][$property->name]['literal'] = (! empty($annotation->literal))
                             ? $annotation->literal
                             : $annotation->value;
                     }
@@ -589,6 +610,13 @@ final class DocParser
 
                 // choose the first property as default property
                 $metadata['default_property'] = reset($metadata['properties']);
+            } elseif ($metadata['has_named_argument_constructor']) {
+                foreach ($constructor->getParameters() as $parameter) {
+                    $metadata['constructor_args'][$parameter->getName()] = [
+                        'position' => $parameter->getPosition(),
+                        'default' => $parameter->isOptional() ? $parameter->getDefaultValue() : null,
+                    ];
+                }
             }
         }
 
@@ -838,7 +866,8 @@ EXCEPTION
             );
         }
 
-        $values = $this->MethodCall();
+        $arguments = $this->MethodCall();
+        $values    = $this->resolvePositionalValues($arguments, $name);
 
         if (isset(self::$annotationMetadata[$name]['enum'])) {
             // checks all declared attributes
@@ -910,7 +939,7 @@ EXCEPTION
             }
         }
 
-        if (is_subclass_of($name, NamedArgumentConstructorAnnotation::class)) {
+        if (self::$annotationMetadata[$name]['has_named_argument_constructor']) {
             if (PHP_VERSION_ID >= 80000) {
                 return new $name(...$values);
             }
@@ -1033,30 +1062,20 @@ EXCEPTION
             $token = $this->lexer->lookahead;
             $value = $this->Value();
 
-            if (! is_object($value) && ! is_array($value)) {
-                throw $this->syntaxError('Value', $token);
-            }
-
             $values[] = $value;
         }
 
+        $namedArguments      = [];
+        $positionalArguments = [];
         foreach ($values as $k => $value) {
             if (is_object($value) && $value instanceof stdClass) {
-                $values[$value->name] = $value->value;
-            } elseif (! isset($values['value'])) {
-                $values['value'] = $value;
+                $namedArguments[$value->name] = $value->value;
             } else {
-                if (! is_array($values['value'])) {
-                    $values['value'] = [$values['value']];
-                }
-
-                $values['value'][] = $value;
+                $positionalArguments[$k] = $value;
             }
-
-            unset($values[$k]);
         }
 
-        return $values;
+        return ['named_arguments' => $namedArguments, 'positional_arguments' => $positionalArguments];
     }
 
     /**
@@ -1082,9 +1101,9 @@ EXCEPTION
                 case ! empty($this->namespaces):
                     foreach ($this->namespaces as $ns) {
                         if (class_exists($ns . '\\' . $className) || interface_exists($ns . '\\' . $className)) {
-                             $className = $ns . '\\' . $className;
-                             $found     = true;
-                             break;
+                            $className = $ns . '\\' . $className;
+                            $found     = true;
+                            break;
                         }
                     }
 
@@ -1111,7 +1130,7 @@ EXCEPTION
             }
 
             if ($found) {
-                 $identifier = $className . '::' . $const;
+                $identifier = $className . '::' . $const;
             }
         }
 
@@ -1384,4 +1403,57 @@ EXCEPTION
 
         return false;
     }
+
+    /**
+     * Resolve positional arguments (without name) to named ones
+     *
+     * @param array<string,mixed> $arguments
+     *
+     * @return array<string,mixed>
+     */
+    private function resolvePositionalValues(array $arguments, string $name): array
+    {
+        $positionalArguments = $arguments['positional_arguments'] ?? [];
+        $values              = $arguments['named_arguments'] ?? [];
+
+        if (
+            self::$annotationMetadata[$name]['has_named_argument_constructor']
+            && self::$annotationMetadata[$name]['default_property'] !== null
+        ) {
+            // We must ensure that we don't have positional arguments after named ones
+            $positions    = array_keys($positionalArguments);
+            $lastPosition = null;
+            foreach ($positions as $position) {
+                if (
+                    ($lastPosition === null && $position !== 0) ||
+                    ($lastPosition !== null && $position !== $lastPosition + 1)
+                ) {
+                    throw $this->syntaxError('Positional arguments after named arguments is not allowed');
+                }
+
+                $lastPosition = $position;
+            }
+
+            foreach (self::$annotationMetadata[$name]['constructor_args'] as $property => $parameter) {
+                $position = $parameter['position'];
+                if (isset($values[$property]) || ! isset($positionalArguments[$position])) {
+                    continue;
+                }
+
+                $values[$property] = $positionalArguments[$position];
+            }
+        } else {
+            if (count($positionalArguments) > 0 && ! isset($values['value'])) {
+                if (count($positionalArguments) === 1) {
+                    $value = array_pop($positionalArguments);
+                } else {
+                    $value = array_values($positionalArguments);
+                }
+
+                $values['value'] = $value;
+            }
+        }
+
+        return $values;
+    }
 }
