@@ -3,6 +3,7 @@
 namespace Doctrine\Tests\Common\Annotations;
 
 use Doctrine\Common\Annotations\Annotation;
+use Doctrine\Common\Annotations\Annotation\NamedArgumentConstructor;
 use Doctrine\Common\Annotations\Annotation\Target;
 use Doctrine\Common\Annotations\AnnotationException;
 use Doctrine\Common\Annotations\AnnotationRegistry;
@@ -86,6 +87,33 @@ class DocParserTest extends TestCase
         self::assertInstanceOf(Name::class, $annot->value[0]);
         self::assertInstanceOf(Name::class, $annot->value[1]);
 
+        // Positionals arguments following named arguments
+        $result = $parser->parse('@Name(foo="bar", @Name)');
+        $annot  = $result[0];
+
+        self::assertInstanceOf(Name::class, $annot);
+        self::assertEquals('bar', $annot->foo);
+        self::assertInstanceOf(Name::class, $annot->value);
+
+        // Multiple positionals arguments following named arguments
+        $result = $parser->parse('@Name(@Name, foo="baz", @Name)');
+        $annot  = $result[0];
+
+        self::assertInstanceOf(Name::class, $annot);
+        self::assertEquals('baz', $annot->foo);
+        self::assertIsArray($annot->value);
+        self::assertInstanceOf(Name::class, $annot->value[0]);
+        self::assertInstanceOf(Name::class, $annot->value[1]);
+
+        // Multiple scalar values
+        $result = $parser->parse('@Name("foo", "bar")');
+        $annot  = $result[0];
+
+        self::assertInstanceOf(Name::class, $annot);
+        self::assertIsArray($annot->value);
+        self::assertEquals('foo', $annot->value[0]);
+        self::assertEquals('bar', $annot->value[1]);
+
         // Multiple types as values
         $result = $parser->parse('@Name(foo="Bar", @Name, {"key1"="value1", "key2"="value2"})');
         $annot  = $result[0];
@@ -1074,7 +1102,7 @@ DOCBLOCK;
         $parser->setTarget(Target::TARGET_CLASS);
         $this->expectException(InvalidArgumentException::class);
         $this->expectExceptionMessage(
-            'Invalid Target "Foo". Available targets: [ALL, CLASS, METHOD, PROPERTY, ANNOTATION]'
+            'Invalid Target "Foo". Available targets: [ALL, CLASS, METHOD, PROPERTY, FUNCTION, ANNOTATION]'
         );
         $parser->parse($docblock, $context);
     }
@@ -1539,7 +1567,7 @@ DOCBLOCK;
         self::assertInstanceOf(SomeAnnotationClassNameWithoutConstructorAndProperties::class, $result[0]);
     }
 
-    public function testNamedArgumentsConstructorAnnotation(): void
+    public function testNamedArgumentsConstructorInterface(): void
     {
         $result = $this
             ->createTestParser()
@@ -1551,7 +1579,7 @@ DOCBLOCK;
         self::assertSame(2222, $result[0]->getBar());
     }
 
-    public function testNamedReorderedArgumentsConstructorAnnotation(): void
+    public function testNamedReorderedArgumentsConstructorInterface(): void
     {
         $result = $this
             ->createTestParser()
@@ -1563,7 +1591,7 @@ DOCBLOCK;
         self::assertSame(2222, $result[0]->getBar());
     }
 
-    public function testNamedArgumentsConstructorAnnotationWithDefaultValue(): void
+    public function testNamedArgumentsConstructorInterfaceWithDefaultValue(): void
     {
         $result = $this
             ->createTestParser()
@@ -1574,6 +1602,87 @@ DOCBLOCK;
         self::assertSame('baz', $result[0]->getFoo());
         self::assertSame(1234, $result[0]->getBar());
     }
+
+    public function testNamedArgumentsConstructorAnnotation(): void
+    {
+        $result = $this
+            ->createTestParser()
+            ->parse('/** @AnotherNamedAnnotation(foo="baz", bar=2222) */');
+
+        self::assertCount(1, $result);
+        self::assertInstanceOf(AnotherNamedAnnotation::class, $result[0]);
+        self::assertSame('baz', $result[0]->getFoo());
+        self::assertSame(2222, $result[0]->getBar());
+    }
+
+    public function testNamedReorderedArgumentsConstructorAnnotation(): void
+    {
+        $result = $this
+            ->createTestParser()
+            ->parse('/** @AnotherNamedAnnotation(bar=2222, foo="baz") */');
+
+        self::assertCount(1, $result);
+        self::assertInstanceOf(AnotherNamedAnnotation::class, $result[0]);
+        self::assertSame('baz', $result[0]->getFoo());
+        self::assertSame(2222, $result[0]->getBar());
+    }
+
+    public function testNamedArgumentsConstructorAnnotationWithDefaultValue(): void
+    {
+        $result = $this
+            ->createTestParser()
+            ->parse('/** @AnotherNamedAnnotation(foo="baz") */');
+
+        self::assertCount(1, $result);
+        self::assertInstanceOf(AnotherNamedAnnotation::class, $result[0]);
+        self::assertSame('baz', $result[0]->getFoo());
+        self::assertSame(1234, $result[0]->getBar());
+    }
+
+    public function testNamedArgumentsConstructorAnnotationWithDefaultProperty(): void
+    {
+        $result = $this
+            ->createTestParser()
+            ->parse('/** @AnotherNamedAnnotation("baz") */');
+
+        self::assertCount(1, $result);
+        self::assertInstanceOf(AnotherNamedAnnotation::class, $result[0]);
+        self::assertSame('baz', $result[0]->getFoo());
+        self::assertSame(1234, $result[0]->getBar());
+    }
+
+    public function testNamedArgumentsConstructorAnnotationWithDefaultPropertyAsArray(): void
+    {
+        $result = $this
+            ->createTestParser()
+            ->parse('/** @NamedAnnotationWithArray({"foo","bar","baz"},bar=567) */');
+
+        self::assertCount(1, $result);
+        self::assertInstanceOf(NamedAnnotationWithArray::class, $result[0]);
+        self::assertSame(['foo', 'bar', 'baz'], $result[0]->getFoo());
+        self::assertSame(567, $result[0]->getBar());
+    }
+
+    public function testNamedArgumentsConstructorAnnotationWithDefaultPropertySet(): void
+    {
+        $result = $this
+            ->createTestParser()
+            ->parse('/** @AnotherNamedAnnotation("baz", foo="bar") */');
+
+        self::assertCount(1, $result);
+        self::assertInstanceOf(AnotherNamedAnnotation::class, $result[0]);
+        self::assertSame('bar', $result[0]->getFoo());
+    }
+
+    public function testNamedArgumentsConstructorAnnotationWithInvalidArguments(): void
+    {
+        $parser = $this->createTestParser();
+        $this->expectException(AnnotationException::class);
+        $this->expectExceptionMessage(
+            '[Syntax Error] Expected Positional arguments after named arguments is not allowed'
+        );
+        $parser->parse('/** @AnotherNamedAnnotation("foo", bar=666, "hey") */');
+    }
 }
 
 /** @Annotation */
@@ -1601,6 +1710,74 @@ class NamedAnnotation implements NamedArgumentConstructorAnnotation
     }
 }
 
+/**
+ * @Annotation
+ * @NamedArgumentConstructor
+ */
+class AnotherNamedAnnotation
+{
+    /** @var string */
+    private $foo;
+    /** @var int */
+    private $bar;
+    /** @var string */
+    private $baz;
+
+    public function __construct(string $foo, int $bar = 1234, string $baz = 'baz')
+    {
+        $this->foo = $foo;
+        $this->bar = $bar;
+        $this->baz = $baz;
+    }
+
+    public function getFoo(): string
+    {
+        return $this->foo;
+    }
+
+    public function getBar(): int
+    {
+        return $this->bar;
+    }
+
+    public function getBaz(): string
+    {
+        return $this->baz;
+    }
+}
+
+/**
+ * @Annotation
+ * @NamedArgumentConstructor
+ */
+class NamedAnnotationWithArray
+{
+    /** @var mixed[] */
+    private $foo;
+    /** @var int */
+    private $bar;
+
+    /**
+     * @param mixed[] $foo
+     */
+    public function __construct(array $foo, int $bar = 1234)
+    {
+        $this->foo = $foo;
+        $this->bar = $bar;
+    }
+
+    /** @return mixed[] */
+    public function getFoo(): array
+    {
+        return $this->foo;
+    }
+
+    public function getBar(): int
+    {
+        return $this->bar;
+    }
+}
+
 /** @Annotation */
 class SettingsAnnotation
 {
